# noinspection PyPep8
# noinspection PyArgumentList

"""
AUTO-GENERATED BY `scripts/generate_protocol.py` using `data/browser_protocol.json`
and `data/js_protocol.json` as inputs! Please do not modify this file.
"""

import logging
from typing import Any, Optional, Union

from chromewhip.helpers import PayloadMixin, BaseEvent, ChromeTypeBase

log = logging.getLogger(__name__)

# GraphObjectId: An unique ID for a graph object (AudioContext, AudioNode, AudioParam) in Web Audio API
GraphObjectId = str

# ContextType: Enum of BaseAudioContext types
ContextType = str

# ContextState: Enum of AudioContextState from the spec
ContextState = str

# NodeType: Enum of AudioNode types
NodeType = str

# ChannelCountMode: Enum of AudioNode::ChannelCountMode from the spec
ChannelCountMode = str

# ChannelInterpretation: Enum of AudioNode::ChannelInterpretation from the spec
ChannelInterpretation = str

# ParamType: Enum of AudioParam types
ParamType = str

# AutomationRate: Enum of AudioParam::AutomationRate from the spec
AutomationRate = str

# ContextRealtimeData: Fields in AudioContext that change in real-time.
class ContextRealtimeData(ChromeTypeBase):
    def __init__(self,
                 currentTime: Union['float'],
                 renderCapacity: Union['float'],
                 callbackIntervalMean: Union['float'],
                 callbackIntervalVariance: Union['float'],
                 ):

        self.currentTime = currentTime
        self.renderCapacity = renderCapacity
        self.callbackIntervalMean = callbackIntervalMean
        self.callbackIntervalVariance = callbackIntervalVariance


# BaseAudioContext: Protocol object for BaseAudioContext
class BaseAudioContext(ChromeTypeBase):
    def __init__(self,
                 contextId: Union['GraphObjectId'],
                 contextType: Union['ContextType'],
                 contextState: Union['ContextState'],
                 callbackBufferSize: Union['float'],
                 maxOutputChannelCount: Union['float'],
                 sampleRate: Union['float'],
                 realtimeData: Optional['ContextRealtimeData'] = None,
                 ):

        self.contextId = contextId
        self.contextType = contextType
        self.contextState = contextState
        self.realtimeData = realtimeData
        self.callbackBufferSize = callbackBufferSize
        self.maxOutputChannelCount = maxOutputChannelCount
        self.sampleRate = sampleRate


# AudioListener: Protocol object for AudioListner
class AudioListener(ChromeTypeBase):
    def __init__(self,
                 listenerId: Union['GraphObjectId'],
                 contextId: Union['GraphObjectId'],
                 ):

        self.listenerId = listenerId
        self.contextId = contextId


# AudioNode: Protocol object for AudioNode
class AudioNode(ChromeTypeBase):
    def __init__(self,
                 nodeId: Union['GraphObjectId'],
                 contextId: Union['GraphObjectId'],
                 nodeType: Union['NodeType'],
                 numberOfInputs: Union['float'],
                 numberOfOutputs: Union['float'],
                 channelCount: Union['float'],
                 channelCountMode: Union['ChannelCountMode'],
                 channelInterpretation: Union['ChannelInterpretation'],
                 ):

        self.nodeId = nodeId
        self.contextId = contextId
        self.nodeType = nodeType
        self.numberOfInputs = numberOfInputs
        self.numberOfOutputs = numberOfOutputs
        self.channelCount = channelCount
        self.channelCountMode = channelCountMode
        self.channelInterpretation = channelInterpretation


# AudioParam: Protocol object for AudioParam
class AudioParam(ChromeTypeBase):
    def __init__(self,
                 paramId: Union['GraphObjectId'],
                 nodeId: Union['GraphObjectId'],
                 contextId: Union['GraphObjectId'],
                 paramType: Union['ParamType'],
                 rate: Union['AutomationRate'],
                 defaultValue: Union['float'],
                 minValue: Union['float'],
                 maxValue: Union['float'],
                 ):

        self.paramId = paramId
        self.nodeId = nodeId
        self.contextId = contextId
        self.paramType = paramType
        self.rate = rate
        self.defaultValue = defaultValue
        self.minValue = minValue
        self.maxValue = maxValue


class WebAudio(PayloadMixin):
    """ This domain allows inspection of Web Audio API.
https://webaudio.github.io/web-audio-api/
    """
    @classmethod
    def enable(cls):
        """Enables the WebAudio domain and starts sending context lifetime events.
        """
        return (
            cls.build_send_payload("enable", {
            }),
            None
        )

    @classmethod
    def disable(cls):
        """Disables the WebAudio domain.
        """
        return (
            cls.build_send_payload("disable", {
            }),
            None
        )

    @classmethod
    def getRealtimeData(cls,
                        contextId: Union['GraphObjectId'],
                        ):
        """Fetch the realtime data from the registered contexts.
        :param contextId: 
        :type contextId: GraphObjectId
        """
        return (
            cls.build_send_payload("getRealtimeData", {
                "contextId": contextId,
            }),
            cls.convert_payload({
                "realtimeData": {
                    "class": ContextRealtimeData,
                    "optional": False
                },
            })
        )



class ContextCreatedEvent(BaseEvent):

    js_name = 'Webaudio.contextCreated'
    hashable = []
    is_hashable = False

    def __init__(self,
                 context: Union['BaseAudioContext', dict],
                 ):
        if isinstance(context, dict):
            context = BaseAudioContext(**context)
        self.context = context

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class ContextWillBeDestroyedEvent(BaseEvent):

    js_name = 'Webaudio.contextWillBeDestroyed'
    hashable = ['contextId']
    is_hashable = True

    def __init__(self,
                 contextId: Union['GraphObjectId', dict],
                 ):
        if isinstance(contextId, dict):
            contextId = GraphObjectId(**contextId)
        self.contextId = contextId

    @classmethod
    def build_hash(cls, contextId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class ContextChangedEvent(BaseEvent):

    js_name = 'Webaudio.contextChanged'
    hashable = []
    is_hashable = False

    def __init__(self,
                 context: Union['BaseAudioContext', dict],
                 ):
        if isinstance(context, dict):
            context = BaseAudioContext(**context)
        self.context = context

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class AudioListenerCreatedEvent(BaseEvent):

    js_name = 'Webaudio.audioListenerCreated'
    hashable = []
    is_hashable = False

    def __init__(self,
                 listener: Union['AudioListener', dict],
                 ):
        if isinstance(listener, dict):
            listener = AudioListener(**listener)
        self.listener = listener

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class AudioListenerWillBeDestroyedEvent(BaseEvent):

    js_name = 'Webaudio.audioListenerWillBeDestroyed'
    hashable = ['listenerId', 'contextId']
    is_hashable = True

    def __init__(self,
                 contextId: Union['GraphObjectId', dict],
                 listenerId: Union['GraphObjectId', dict],
                 ):
        if isinstance(contextId, dict):
            contextId = GraphObjectId(**contextId)
        self.contextId = contextId
        if isinstance(listenerId, dict):
            listenerId = GraphObjectId(**listenerId)
        self.listenerId = listenerId

    @classmethod
    def build_hash(cls, listenerId, contextId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class AudioNodeCreatedEvent(BaseEvent):

    js_name = 'Webaudio.audioNodeCreated'
    hashable = []
    is_hashable = False

    def __init__(self,
                 node: Union['AudioNode', dict],
                 ):
        if isinstance(node, dict):
            node = AudioNode(**node)
        self.node = node

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class AudioNodeWillBeDestroyedEvent(BaseEvent):

    js_name = 'Webaudio.audioNodeWillBeDestroyed'
    hashable = ['nodeId', 'contextId']
    is_hashable = True

    def __init__(self,
                 contextId: Union['GraphObjectId', dict],
                 nodeId: Union['GraphObjectId', dict],
                 ):
        if isinstance(contextId, dict):
            contextId = GraphObjectId(**contextId)
        self.contextId = contextId
        if isinstance(nodeId, dict):
            nodeId = GraphObjectId(**nodeId)
        self.nodeId = nodeId

    @classmethod
    def build_hash(cls, nodeId, contextId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class AudioParamCreatedEvent(BaseEvent):

    js_name = 'Webaudio.audioParamCreated'
    hashable = []
    is_hashable = False

    def __init__(self,
                 param: Union['AudioParam', dict],
                 ):
        if isinstance(param, dict):
            param = AudioParam(**param)
        self.param = param

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')


class AudioParamWillBeDestroyedEvent(BaseEvent):

    js_name = 'Webaudio.audioParamWillBeDestroyed'
    hashable = ['nodeId', 'paramId', 'contextId']
    is_hashable = True

    def __init__(self,
                 contextId: Union['GraphObjectId', dict],
                 nodeId: Union['GraphObjectId', dict],
                 paramId: Union['GraphObjectId', dict],
                 ):
        if isinstance(contextId, dict):
            contextId = GraphObjectId(**contextId)
        self.contextId = contextId
        if isinstance(nodeId, dict):
            nodeId = GraphObjectId(**nodeId)
        self.nodeId = nodeId
        if isinstance(paramId, dict):
            paramId = GraphObjectId(**paramId)
        self.paramId = paramId

    @classmethod
    def build_hash(cls, nodeId, paramId, contextId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class NodesConnectedEvent(BaseEvent):

    js_name = 'Webaudio.nodesConnected'
    hashable = ['sourceId', 'contextId', 'destinationId']
    is_hashable = True

    def __init__(self,
                 contextId: Union['GraphObjectId', dict],
                 sourceId: Union['GraphObjectId', dict],
                 destinationId: Union['GraphObjectId', dict],
                 sourceOutputIndex: Union['float', dict, None] = None,
                 destinationInputIndex: Union['float', dict, None] = None,
                 ):
        if isinstance(contextId, dict):
            contextId = GraphObjectId(**contextId)
        self.contextId = contextId
        if isinstance(sourceId, dict):
            sourceId = GraphObjectId(**sourceId)
        self.sourceId = sourceId
        if isinstance(destinationId, dict):
            destinationId = GraphObjectId(**destinationId)
        self.destinationId = destinationId
        if isinstance(sourceOutputIndex, dict):
            sourceOutputIndex = float(**sourceOutputIndex)
        self.sourceOutputIndex = sourceOutputIndex
        if isinstance(destinationInputIndex, dict):
            destinationInputIndex = float(**destinationInputIndex)
        self.destinationInputIndex = destinationInputIndex

    @classmethod
    def build_hash(cls, sourceId, contextId, destinationId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class NodesDisconnectedEvent(BaseEvent):

    js_name = 'Webaudio.nodesDisconnected'
    hashable = ['sourceId', 'contextId', 'destinationId']
    is_hashable = True

    def __init__(self,
                 contextId: Union['GraphObjectId', dict],
                 sourceId: Union['GraphObjectId', dict],
                 destinationId: Union['GraphObjectId', dict],
                 sourceOutputIndex: Union['float', dict, None] = None,
                 destinationInputIndex: Union['float', dict, None] = None,
                 ):
        if isinstance(contextId, dict):
            contextId = GraphObjectId(**contextId)
        self.contextId = contextId
        if isinstance(sourceId, dict):
            sourceId = GraphObjectId(**sourceId)
        self.sourceId = sourceId
        if isinstance(destinationId, dict):
            destinationId = GraphObjectId(**destinationId)
        self.destinationId = destinationId
        if isinstance(sourceOutputIndex, dict):
            sourceOutputIndex = float(**sourceOutputIndex)
        self.sourceOutputIndex = sourceOutputIndex
        if isinstance(destinationInputIndex, dict):
            destinationInputIndex = float(**destinationInputIndex)
        self.destinationInputIndex = destinationInputIndex

    @classmethod
    def build_hash(cls, sourceId, contextId, destinationId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class NodeParamConnectedEvent(BaseEvent):

    js_name = 'Webaudio.nodeParamConnected'
    hashable = ['sourceId', 'contextId', 'destinationId']
    is_hashable = True

    def __init__(self,
                 contextId: Union['GraphObjectId', dict],
                 sourceId: Union['GraphObjectId', dict],
                 destinationId: Union['GraphObjectId', dict],
                 sourceOutputIndex: Union['float', dict, None] = None,
                 ):
        if isinstance(contextId, dict):
            contextId = GraphObjectId(**contextId)
        self.contextId = contextId
        if isinstance(sourceId, dict):
            sourceId = GraphObjectId(**sourceId)
        self.sourceId = sourceId
        if isinstance(destinationId, dict):
            destinationId = GraphObjectId(**destinationId)
        self.destinationId = destinationId
        if isinstance(sourceOutputIndex, dict):
            sourceOutputIndex = float(**sourceOutputIndex)
        self.sourceOutputIndex = sourceOutputIndex

    @classmethod
    def build_hash(cls, sourceId, contextId, destinationId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class NodeParamDisconnectedEvent(BaseEvent):

    js_name = 'Webaudio.nodeParamDisconnected'
    hashable = ['sourceId', 'contextId', 'destinationId']
    is_hashable = True

    def __init__(self,
                 contextId: Union['GraphObjectId', dict],
                 sourceId: Union['GraphObjectId', dict],
                 destinationId: Union['GraphObjectId', dict],
                 sourceOutputIndex: Union['float', dict, None] = None,
                 ):
        if isinstance(contextId, dict):
            contextId = GraphObjectId(**contextId)
        self.contextId = contextId
        if isinstance(sourceId, dict):
            sourceId = GraphObjectId(**sourceId)
        self.sourceId = sourceId
        if isinstance(destinationId, dict):
            destinationId = GraphObjectId(**destinationId)
        self.destinationId = destinationId
        if isinstance(sourceOutputIndex, dict):
            sourceOutputIndex = float(**sourceOutputIndex)
        self.sourceOutputIndex = sourceOutputIndex

    @classmethod
    def build_hash(cls, sourceId, contextId, destinationId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h



# noinspection PyPep8
# noinspection PyArgumentList

"""
AUTO-GENERATED BY `scripts/generate_protocol.py` using `data/browser_protocol.json`
and `data/js_protocol.json` as inputs! Please do not modify this file.
"""

import logging
from typing import Any, Optional, Union

from chromewhip.helpers import PayloadMixin, BaseEvent, ChromeTypeBase

log = logging.getLogger(__name__)

# PlayerId: Players will get an ID that is unique within the agent context.
PlayerId = str

# Timestamp: 
Timestamp = float

# PlayerProperty: Player Property type
class PlayerProperty(ChromeTypeBase):
    def __init__(self,
                 name: Union['str'],
                 value: Optional['str'] = None,
                 ):

        self.name = name
        self.value = value


# PlayerEventType: Break out events into different types
PlayerEventType = str

# PlayerEvent: 
class PlayerEvent(ChromeTypeBase):
    def __init__(self,
                 type: Union['PlayerEventType'],
                 timestamp: Union['Timestamp'],
                 name: Union['str'],
                 value: Union['str'],
                 ):

        self.type = type
        self.timestamp = timestamp
        self.name = name
        self.value = value


class Media(PayloadMixin):
    """ This domain allows detailed inspection of media elements
    """
    @classmethod
    def enable(cls):
        """Enables the Media domain
        """
        return (
            cls.build_send_payload("enable", {
            }),
            None
        )

    @classmethod
    def disable(cls):
        """Disables the Media domain.
        """
        return (
            cls.build_send_payload("disable", {
            }),
            None
        )



class PlayerPropertiesChangedEvent(BaseEvent):

    js_name = 'Media.playerPropertiesChanged'
    hashable = ['playerId']
    is_hashable = True

    def __init__(self,
                 playerId: Union['PlayerId', dict],
                 properties: Union['[PlayerProperty]', dict],
                 ):
        if isinstance(playerId, dict):
            playerId = PlayerId(**playerId)
        self.playerId = playerId
        if isinstance(properties, dict):
            properties = [PlayerProperty](**properties)
        self.properties = properties

    @classmethod
    def build_hash(cls, playerId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class PlayerEventsAddedEvent(BaseEvent):

    js_name = 'Media.playerEventsAdded'
    hashable = ['playerId']
    is_hashable = True

    def __init__(self,
                 playerId: Union['PlayerId', dict],
                 events: Union['[PlayerEvent]', dict],
                 ):
        if isinstance(playerId, dict):
            playerId = PlayerId(**playerId)
        self.playerId = playerId
        if isinstance(events, dict):
            events = [PlayerEvent](**events)
        self.events = events

    @classmethod
    def build_hash(cls, playerId):
        kwargs = locals()
        kwargs.pop('cls')
        serialized_id_params = ','.join(['='.join([p, str(v)]) for p, v in kwargs.items()])
        h = '{}:{}'.format(cls.js_name, serialized_id_params)
        log.debug('generated hash = %s' % h)
        return h


class PlayersCreatedEvent(BaseEvent):

    js_name = 'Media.playersCreated'
    hashable = []
    is_hashable = False

    def __init__(self,
                 players: Union['[PlayerId]', dict],
                 ):
        if isinstance(players, dict):
            players = [PlayerId](**players)
        self.players = players

    @classmethod
    def build_hash(cls):
        raise ValueError('Unable to build hash for non-hashable type')



# noinspection PyPep8
# noinspection PyArgumentList

"""
AUTO-GENERATED BY `scripts/generate_protocol.py` using `data/browser_protocol.json`
and `data/js_protocol.json` as inputs! Please do not modify this file.
"""

import logging
from typing import Any, Optional, Union

from chromewhip.helpers import PayloadMixin, BaseEvent, ChromeTypeBase

log = logging.getLogger(__name__)

# GPUDevice: Describes a single graphics processor (GPU).
class GPUDevice(ChromeTypeBase):
    def __init__(self,
                 vendorId: Union['float'],
                 deviceId: Union['float'],
                 vendorString: Union['str'],
                 deviceString: Union['str'],
                 driverVendor: Union['str'],
                 driverVersion: Union['str'],
                 ):

        self.vendorId = vendorId
        self.deviceId = deviceId
        self.vendorString = vendorString
        self.deviceString = deviceString
        self.driverVendor = driverVendor
        self.driverVersion = driverVersion


# Size: Describes the width and height dimensions of an entity.
class Size(ChromeTypeBase):
    def __init__(self,
                 width: Union['int'],
                 height: Union['int'],
                 ):

        self.width = width
        self.height = height


# VideoDecodeAcceleratorCapability: Describes a supported video decoding profile with its associated minimum andmaximum resolutions.
class VideoDecodeAcceleratorCapability(ChromeTypeBase):
    def __init__(self,
                 profile: Union['str'],
                 maxResolution: Union['Size'],
                 minResolution: Union['Size'],
                 ):

        self.profile = profile
        self.maxResolution = maxResolution
        self.minResolution = minResolution


# VideoEncodeAcceleratorCapability: Describes a supported video encoding profile with its associated maximumresolution and maximum framerate.
class VideoEncodeAcceleratorCapability(ChromeTypeBase):
    def __init__(self,
                 profile: Union['str'],
                 maxResolution: Union['Size'],
                 maxFramerateNumerator: Union['int'],
                 maxFramerateDenominator: Union['int'],
                 ):

        self.profile = profile
        self.maxResolution = maxResolution
        self.maxFramerateNumerator = maxFramerateNumerator
        self.maxFramerateDenominator = maxFramerateDenominator


# SubsamplingFormat: YUV subsampling type of the pixels of a given image.
SubsamplingFormat = str

# ImageType: Image format of a given image.
ImageType = str

# ImageDecodeAcceleratorCapability: Describes a supported image decoding profile with its associated minimum andmaximum resolutions and subsampling.
class ImageDecodeAcceleratorCapability(ChromeTypeBase):
    def __init__(self,
                 imageType: Union['ImageType'],
                 maxDimensions: Union['Size'],
                 minDimensions: Union['Size'],
                 subsamplings: Union['[SubsamplingFormat]'],
                 ):

        self.imageType = imageType
        self.maxDimensions = maxDimensions
        self.minDimensions = minDimensions
        self.subsamplings = subsamplings


# GPUInfo: Provides information about the GPU(s) on the system.
class GPUInfo(ChromeTypeBase):
    def __init__(self,
                 devices: Union['[GPUDevice]'],
                 driverBugWorkarounds: Union['[]'],
                 videoDecoding: Union['[VideoDecodeAcceleratorCapability]'],
                 videoEncoding: Union['[VideoEncodeAcceleratorCapability]'],
                 imageDecoding: Union['[ImageDecodeAcceleratorCapability]'],
                 auxAttributes: Optional['dict'] = None,
                 featureStatus: Optional['dict'] = None,
                 ):

        self.devices = devices
        self.auxAttributes = auxAttributes
        self.featureStatus = featureStatus
        self.driverBugWorkarounds = driverBugWorkarounds
        self.videoDecoding = videoDecoding
        self.videoEncoding = videoEncoding
        self.imageDecoding = imageDecoding


# ProcessInfo: Represents process info.
class ProcessInfo(ChromeTypeBase):
    def __init__(self,
                 type: Union['str'],
                 id: Union['int'],
                 cpuTime: Union['float'],
                 ):

        self.type = type
        self.id = id
        self.cpuTime = cpuTime


class SystemInfo(PayloadMixin):
    """ The SystemInfo domain defines methods and events for querying low-level system information.
    """
    @classmethod
    def getInfo(cls):
        """Returns information about the system.
        """
        return (
            cls.build_send_payload("getInfo", {
            }),
            cls.convert_payload({
                "gpu": {
                    "class": GPUInfo,
                    "optional": False
                },
                "modelName": {
                    "class": str,
                    "optional": False
                },
                "modelVersion": {
                    "class": str,
                    "optional": False
                },
                "commandLine": {
                    "class": str,
                    "optional": False
                },
            })
        )

    @classmethod
    def getProcessInfo(cls):
        """Returns information about all running processes.
        """
        return (
            cls.build_send_payload("getProcessInfo", {
            }),
            cls.convert_payload({
                "processInfo": {
                    "class": [ProcessInfo],
                    "optional": False
                },
            })
        )




# noinspection PyPep8
# noinspection PyArgumentList

"""
AUTO-GENERATED BY `scripts/generate_protocol.py` using `data/browser_protocol.json`
and `data/js_protocol.json` as inputs! Please do not modify this file.
"""

import logging
from typing import Any, Optional, Union

from chromewhip.helpers import PayloadMixin, BaseEvent, ChromeTypeBase

log = logging.getLogger(__name__)

# AuthenticatorId: 
AuthenticatorId = str

# AuthenticatorProtocol: 
AuthenticatorProtocol = str

# AuthenticatorTransport: 
AuthenticatorTransport = str

# VirtualAuthenticatorOptions: 
class VirtualAuthenticatorOptions(ChromeTypeBase):
    def __init__(self,
                 protocol: Union['AuthenticatorProtocol'],
                 transport: Union['AuthenticatorTransport'],
                 hasResidentKey: Union['bool'],
                 hasUserVerification: Union['bool'],
                 automaticPresenceSimulation: Optional['bool'] = None,
                 ):

        self.protocol = protocol
        self.transport = transport
        self.hasResidentKey = hasResidentKey
        self.hasUserVerification = hasUserVerification
        self.automaticPresenceSimulation = automaticPresenceSimulation


# Credential: 
class Credential(ChromeTypeBase):
    def __init__(self,
                 credentialId: Union['str'],
                 isResidentCredential: Union['bool'],
                 privateKey: Union['str'],
                 signCount: Union['int'],
                 rpId: Optional['str'] = None,
                 userHandle: Optional['str'] = None,
                 ):

        self.credentialId = credentialId
        self.isResidentCredential = isResidentCredential
        self.rpId = rpId
        self.privateKey = privateKey
        self.userHandle = userHandle
        self.signCount = signCount


class WebAuthn(PayloadMixin):
    """ This domain allows configuring virtual authenticators to test the WebAuthn
API.
    """
    @classmethod
    def enable(cls):
        """Enable the WebAuthn domain and start intercepting credential storage and
retrieval with a virtual authenticator.
        """
        return (
            cls.build_send_payload("enable", {
            }),
            None
        )

    @classmethod
    def disable(cls):
        """Disable the WebAuthn domain.
        """
        return (
            cls.build_send_payload("disable", {
            }),
            None
        )

    @classmethod
    def addVirtualAuthenticator(cls,
                                options: Union['VirtualAuthenticatorOptions'],
                                ):
        """Creates and adds a virtual authenticator.
        :param options: 
        :type options: VirtualAuthenticatorOptions
        """
        return (
            cls.build_send_payload("addVirtualAuthenticator", {
                "options": options,
            }),
            cls.convert_payload({
                "authenticatorId": {
                    "class": AuthenticatorId,
                    "optional": False
                },
            })
        )

    @classmethod
    def removeVirtualAuthenticator(cls,
                                   authenticatorId: Union['AuthenticatorId'],
                                   ):
        """Removes the given authenticator.
        :param authenticatorId: 
        :type authenticatorId: AuthenticatorId
        """
        return (
            cls.build_send_payload("removeVirtualAuthenticator", {
                "authenticatorId": authenticatorId,
            }),
            None
        )

    @classmethod
    def addCredential(cls,
                      authenticatorId: Union['AuthenticatorId'],
                      credential: Union['Credential'],
                      ):
        """Adds the credential to the specified authenticator.
        :param authenticatorId: 
        :type authenticatorId: AuthenticatorId
        :param credential: 
        :type credential: Credential
        """
        return (
            cls.build_send_payload("addCredential", {
                "authenticatorId": authenticatorId,
                "credential": credential,
            }),
            None
        )

    @classmethod
    def getCredential(cls,
                      authenticatorId: Union['AuthenticatorId'],
                      credentialId: Union['str'],
                      ):
        """Returns a single credential stored in the given virtual authenticator that
matches the credential ID.
        :param authenticatorId: 
        :type authenticatorId: AuthenticatorId
        :param credentialId: 
        :type credentialId: str
        """
        return (
            cls.build_send_payload("getCredential", {
                "authenticatorId": authenticatorId,
                "credentialId": credentialId,
            }),
            cls.convert_payload({
                "credential": {
                    "class": Credential,
                    "optional": False
                },
            })
        )

    @classmethod
    def getCredentials(cls,
                       authenticatorId: Union['AuthenticatorId'],
                       ):
        """Returns all the credentials stored in the given virtual authenticator.
        :param authenticatorId: 
        :type authenticatorId: AuthenticatorId
        """
        return (
            cls.build_send_payload("getCredentials", {
                "authenticatorId": authenticatorId,
            }),
            cls.convert_payload({
                "credentials": {
                    "class": [Credential],
                    "optional": False
                },
            })
        )

    @classmethod
    def removeCredential(cls,
                         authenticatorId: Union['AuthenticatorId'],
                         credentialId: Union['str'],
                         ):
        """Removes a credential from the authenticator.
        :param authenticatorId: 
        :type authenticatorId: AuthenticatorId
        :param credentialId: 
        :type credentialId: str
        """
        return (
            cls.build_send_payload("removeCredential", {
                "authenticatorId": authenticatorId,
                "credentialId": credentialId,
            }),
            None
        )

    @classmethod
    def clearCredentials(cls,
                         authenticatorId: Union['AuthenticatorId'],
                         ):
        """Clears all the credentials from the specified device.
        :param authenticatorId: 
        :type authenticatorId: AuthenticatorId
        """
        return (
            cls.build_send_payload("clearCredentials", {
                "authenticatorId": authenticatorId,
            }),
            None
        )

    @classmethod
    def setUserVerified(cls,
                        authenticatorId: Union['AuthenticatorId'],
                        isUserVerified: Union['bool'],
                        ):
        """Sets whether User Verification succeeds or fails for an authenticator.
The default is true.
        :param authenticatorId: 
        :type authenticatorId: AuthenticatorId
        :param isUserVerified: 
        :type isUserVerified: bool
        """
        return (
            cls.build_send_payload("setUserVerified", {
                "authenticatorId": authenticatorId,
                "isUserVerified": isUserVerified,
            }),
            None
        )

